/* This script reimplements the sword blank and related functions, to allow you to craft custom swords. */

//Constant definitions
//--------------------

//These qualities control the appearance and readiness of the regular swordblank, and are set by temperSword() when the sword is worked upon the anvil. The numbers essentially mean the number of hits it takes to reach that state, except that there is a high chance that the quality will go up and a low chance it will go down with each hit.
enum swordblank_qualities
{
	SWORDBLANK_READY	= 9,	//At this quality, the swordblank is hardened by quenching and is ready to be completed by Menion
	SWORDBLANK_QUENCH	= 8,	//At this quality, the swordblank is ready to be quenched and cannot be hammered any more
								//(If the sword is allowed to cool without quenching, it will reduce to below this quality again)
	SWORDBLANK_IMPROVED	= 4		//At and above this quality, the swordblank has an improved appearance
};

//These control the chance that the swordblank's quality is improved/reduced with every blow of the hammer.
enum swordblank_chances
{
	//The base % chance a hit will mar the blade (final fail chance = BASE_TEMPER_FAIL_CHANCE - (dexterity / 2)
	BASE_TEMPER_FAIL_CHANCE = 20,
	//The base % chance a hit will improve the blade (final succeed chance = BASE_TEMPER_SUCCEED_CHANCE + (strength / 2)
	BASE_TEMPER_SUCCEED_CHANCE = 60
};

//How fast the swordblank cools down (one frame every n ticks)
const int SWORDBLANK_COOL_SPEED = 25;

//The sound of quenching a sword in a water trough
const int SOUND_QUENCH_SWORD = 46;


//Externals (my, there's a lot of them)
//---------

extern startCooling 0xB32();	//Makes the swordblank cool down one notch every SWORDBLANK_COOL_SPEED ticks
extern setCooledFrame 0xB37();	//Sets what appearance the swordblank has when it cools down completely
								//(governed by 3 different quality states, see swordblank_qualities above)

//Lives in anvil.uc: returns true if <swordblank> is correctly positioned on <anvil>, false otherwise
extern onAnvil 0xB33(var swordblank, var anvil);
extern useHammerOnSwordblank 0xB34();	//Lives in hammer.uc: Called when a hammer is used on a heated swordblank

extern useSwordOnAnvil 0x68B();			//Lives in anvil.uc - places swordblank on top of anvil
extern useSwordOnFirepit 0x68C();		//Lives in firepit.uc - places swordblank on top of firepit
extern useSwordOnTrough 0x68E();		//Quenches sword in water trough


//Item-use scripts
//----------------

//returns true if the specified blank is the Black Sword blank, false otherwise
isBlackSword 0xB36(var swordblank)	{ return (UI_get_item_frame(swordblank) >= 8); }

//The swordblank object itself. This can be used on several different objects to automatically place it on those objects (e.g. on a firepit) or perform other forging-related behaviours (e.g. quenching it in a trough of water).
SwordBlank 668()
{
	var current_frame;
	var is_blacksword;

	var anvil;
	var target;
	var target_shape;
	var target_offsetx;
	var target_offsety;
	var swordblank;

	var hot_frames;		//Hot enough to hammer
	var warm_frames;	//Heated, but not hot enough
	var cool_frames;	//Cooled down completely


	current_frame = UI_get_item_frame(item);
	is_blacksword = isBlackSword(item);

	//The swordblank has different behaviours when clicked, depending on what heated state it is in.
	//e.g. if the sword is on an anvil and is hot, doubleclicking it will hit it with a hammer; if it is too cool for hammering, it will be picked up instead.
	if (is_blacksword)	{ hot_frames = [10, 11, 12];	warm_frames = [8, 9];	cool_frames = [13, 14, 15]; }
	else				{ hot_frames = [2, 3, 4, 5];	warm_frames = [1];		cool_frames = [0, 6, 7]; }
	
	if (event == DOUBLECLICK)
	{
		//Avatar is wielding a hammer - now check if the swordblank is sitting on an anvil and is hot enough
		//If so, we will automatically hammer the sword; if not, just pick it up normally
		if (UI_is_readied(AVATAR, WEAPON_HAND, SHAPE_HAMMER, FRAME_ANY))
		{
			anvil = UI_find_nearest(item, SHAPE_ANVIL, 3);
			if (anvil && onAnvil(item, anvil) && current_frame in hot_frames)
			{
				gotoObject(anvil, 0, 2, 0, useHammerOnSwordblank, item, SCRIPTED);
				return;
			}
		}
		//The swordblank is just sitting there, demanding to be picked up. Go and get it, then call this function again as event = SCRIPTED
		gotoAndGet(item);
	}

	//Swordblank is now carried and is ready to be used on something
	else if (event == SCRIPTED)
	{
		UI_close_gumps();
		target = UI_click_on_item();
		target_shape = UI_get_item_shape(target);

		//Now, decide what to do based on the shape of the target item

		//Used on anvil: go to anvil and call this function again with event level 8
		if (target_shape == SHAPE_ANVIL)		gotoObject(target, 0, 2, 0, SwordBlank, item, 8);

		//Used on firepit: go to firepit and call this function again with event level 9
		else if (target_shape == SHAPE_FIREPIT)	gotoObject(target, 1, 0, 0, SwordBlank, item, 9);

		//Used on water trough: if sword is hot enough and trough isn't empty, go to trough and call this function again with event level 10.
		else if (target_shape in [SHAPE_TROUGH_HORIZONTAL, SHAPE_TROUGH_VERTICAL])
		{
			//make sure the trough is full
			if (UI_get_item_frame(target) in [3, 7])
			{
				//now check if the swordblank is hot
				if (current_frame in hot_frames || current_frame in warm_frames)
				{
					//stolen from Bucket()
					if (target_shape == SHAPE_TROUGH_HORIZONTAL)
					{
						target_offsetx = [-1, -2, -1, -2, 1, 1, -4, -4];
						target_offsety = [1, 1, -2, -2, 0, -1, 0, -1];
					}
					else
					{
						target_offsetx = [1, 1, -2, -2, 0, -1, 0, -1];
						target_offsety = [-1, -2, -1, -2, 1, 1, -4, -4];
					}

					//Todo: pull vertical trough offsets out of Bucket()
					gotoObject(target, target_offsetx, target_offsety, 0, SwordBlank, item, 10);
				}
				else avatarBark("@The sword's not hot.@");
			}
			else avatarBark("@There's not enough water.@");
		}
	}

	//Swordblank was used on anvil
	else if (event == 8)
	{
		//Animate the avatar
		script AVATAR
		{
			face NORTH;
			//directionFromAvatar(item);
			actor frame LEAN;
			wait 3;
			actor frame STAND;
		}

		//Do the swordblank's scripting
		script item
		{
			nohalt;
			wait 3;
			call useSwordOnAnvil;	//Place the blank atop the anvil
			call startCooling;		//Since this script will have overridden the original cooling behaviour
		}
	}

	//Swordblank was used on firepit
	else if (event == 9)
	{
		//Animate the avatar
		script AVATAR
		{
			face NORTH;	//This is hardcoded because I can't be bothered finding the nearest firepit just to measure the direction
			//directionFromAvatar(item);
			actor frame LEAN;
			wait 3;
			actor frame STAND;
		}

		//Do the swordblank's scripting
		script item
		{
			nohalt;
			wait 3;
			call useSwordOnFirepit;	//Place the blank atop the firepit
			call startCooling;		//Since this script will have overridden the original cooling behaviour
		}
	}

	//Swordblank was used on trough
	else if (event == 10)
	{
		//Animate the avatar
		script AVATAR
		{
			//face directionFromAvatar(item);
			face WEST;
			actor frame LEAN;
			wait 10;
			frame STAND;
		}

		//Do the swordblank's scripting
		script item
		{
			nohalt;
			wait 5;
			call useSwordOnTrough;
			//No startCooling, since the blade is now cold
		}
	}
}

//Swordblank was used on a trough of water (item is swordblank)
useSwordOnTrough 0x68E()
{
	var trough;
	var swordblank_pos;
	var offsetx;
	var offsety;

	//Find the nearest horizontal/vertical trough we can
	trough = UI_find_nearest(UI_get_avatar_ref(), SHAPE_TROUGH_HORIZONTAL, 5);
	if (!trough) trough = UI_find_nearest(UI_get_avatar_ref(), SHAPE_TROUGH_VERTICAL, 5);
	if (!trough) return;

	//Todo: work out horizontal trough offsets
	if (UI_get_item_shape(trough) == SHAPE_TROUGH_VERTICAL)
	{
		offsetx = 1;
		offsety = -1;
	}
	else
	{
	}

	if (placeOnTarget(item, trough, 1, 0, 2))
	{
		swordblank_pos = UI_get_object_position(item);
		UI_sprite_effect(9, (swordblank_pos[X] - 3), (swordblank_pos[Y] - 3), 0, 0, 0, -1);
		UI_play_sound_effect(SOUND_QUENCH_SWORD);

		if (isBlackSword())
		{
			item.setCooledFrame();
		}
		else
		{
			if (UI_get_item_quality(item) == SWORDBLANK_QUENCH)
			{
				UI_set_item_quality(item, SWORDBLANK_READY);
				AVATAR.say("The water hisses and bubbles as you sink the red-hot swordblank into it.~When you draw it out, the cold blade feels lighter in your hand and has taken on an deadly sheen.");
			}
			item.setCooledFrame();
		}

		//Empty the trough
		script trough	{ repeat 2 { previous frame; }; }
	}
}


//Heating and cooling animation scripts
//-------------------------------------

/* This function controls the heating animation of the swordblank, when it is heated on a firepit. */
heatSwordBlank 0xB30()
{
	var current_frame;
	var is_blacksword;
	var min_frame;
	var max_frame;
	var target_frame;

	var hot_frames;		//Hot enough to hammer
	var warm_frames;	//Heated, but not hot enough
	var cool_frames;	//Cooled down completely

	current_frame = UI_get_item_frame(item);
	is_blacksword = isBlackSword(item);

	//Frames for the Black Sword
	if (is_blacksword)
	{
		hot_frames	= [10, 11, 12];
		warm_frames = [8, 9];
		cool_frames = [13, 14, 15];

		min_frame =	8;	//The minimum heated frame to cool down to
		max_frame = 12;	//The maximum heated frame to heat up to
	}
	//Frames for the regular swordblank
	else
	{
		hot_frames	= [2, 3, 4, 5];
		warm_frames = [1];
		cool_frames = [0, 6, 7];

		min_frame =	1;	//The minimum heated frame to cool down to
		max_frame = 5;	//The maximum heated frame to heat up to
	}

	if (current_frame in cool_frames) target_frame = min_frame;
	else
	{
		target_frame = current_frame + 1;
		if (target_frame > max_frame) target_frame = max_frame;
	}

	//Now, set the swordblank to the target frame then start it cooling down again.
	script item	 { wait 2; frame target_frame; call startCooling; }
}

//This function slowly cools down a hot swordblank to its cool state
//More complex than you might think it needs to be, because it's self-calling and there's special frame-selection when a sword reaches its coolest state.
coolSwordBlank 0xB31()
{
	var current_frame;
	var is_blacksword;
	var hot_frames;
	var warm_frames;
	var cool_frames;
	var min_frame;
	var target_frame;
	var quality;

	current_frame = UI_get_item_frame(item);
	is_blacksword = isBlackSword(item);

	if (is_blacksword)	{ hot_frames = [10, 11, 12];	warm_frames = [8, 9];	cool_frames = [13, 14, 15]; min_frame = 8; }
	else				{ hot_frames = [2, 3, 4, 5];	warm_frames = [1];		cool_frames = [0, 6, 7]; min_frame = 1; }

	//We're on the last heated frame - cool the sword down completely to the appropriate base frame
	if (current_frame == min_frame)
	{
		item.setCooledFrame();

		//If the swordblank was ready to be quenched but was left to cool naturally, it will warp and revert to a lower quality
		if (!is_blacksword)
		{
			quality = UI_get_item_quality(item);
			if (quality == SWORDBLANK_QUENCH)
			{
				UI_set_item_quality(item, quality - 2);
				avatarSay("The blank warps as it cools down -- it will need more heating and work to straighten the blade again.~Once the blade is at its sharpest, it must be quenched quickly in a water trough to strengthen and harden the blade.");
			}
		}
	}

	//Sword still has more cooling to do, decrement the frame and call this function again
	else if (current_frame in hot_frames || current_frame in warm_frames)
	{
		script item
		{
			previous frame;
			call startCooling;
		}
	}
}

//The initial function to start the swordblank cooling down. This should be called *whenever the swordblank is used in a script block*, since that script would otherwise override the cooling-down script and the blank would stay hot forever.
startCooling 0xB32()	{ script item after SWORDBLANK_COOL_SPEED ticks	{ call coolSwordBlank; } }

//Set the base cool frame of the swordblank (this depends on how much it has been worked)
setCooledFrame 0xB37()
{
	var target_frame;
	var quality;

	if (isBlackSword(item))
	{
		if		(gflags[0x32E]) target_frame = 15;
		else if	(gflags[0x32D]) target_frame = 14;
		else					target_frame = 13;
	}
	else
	{
		quality = UI_get_item_quality(item);
		if		(quality == SWORDBLANK_READY)		target_frame = 7;
		else if (quality >= SWORDBLANK_IMPROVED)	target_frame = 6;
		else target_frame = 0;
	}
	UI_set_item_frame(item, target_frame);
}


//Quality control scripts
//-----------------------

//This function is called when the swordblank is worked upon the anvil with a hammer - it raises (or lowers) the quality of the swordblank. The only place this function is called from is useHammerOnSwordblank() in hammer.uc.
temperSword 0xB35()
{
	var swordblank;
	var rand;
	var quality;
	var strength;
	var dexterity;
	var fail_chance;
	var succeed_chance;

	//Find the nearest swordblank to the Avatar
	//(we can't provide swordblank as the <item>, because this would require a script block - and we don't want to override the cooling-down script animation.)
	swordblank = UI_find_nearest(UI_get_avatar_ref(), SHAPE_SWORDBLANK, 3);
	if (!swordblank) return;

	rand = UI_get_random(100);

	//The Black Sword is handled by gflags, for now (stupid though that is)
	if (isBlackSword(swordblank))
	{
		//Sword is at its shittiest state
		if (!gflags[0x32D])
		{
			if (rand > 66)
			{
				gflags[0x32D] = true;
				avatarSay("After a short while you notice that the edge has definitely improved.");
			}
		}

		//Sword has been improved
		else if (!gflags[0x32E])
		{
			if (rand > 66)
			{
				gflags[0x32E] = true;
				gflags[0x337] = true;
				avatarSay("You feel that you've done the best job that you can, but the sword doesn't feel quite right. It's much too heavy and cumbersome to wield as a weapon.");
			}
			//Whoops! There goes your hard work!
			else if (rand < 20)
			{
				gflags[0x32D] = false;
				avatarSay("That last blow was perhaps a bit too hard, It'll take a while to hammer out the flaws.");
			}
		}

		//Sword is as good as it gets
		else
		{
			avatarSay("The blade has been worked as well as it can be. It will take some form of magic to make this sword blank into a usable weapon.");
		}
	}

	//The regular swordblank is driven by qualities instead. Much more sensible!
	else
	{
		quality = UI_get_item_quality(swordblank);

		//Swordblank is already as good as it gets
		if (quality == SWORDBLANK_READY)
		{
			avatarSay("This blade is already sharp and true, and ready to be given to a weaponsmith for the finishing touch - the hilt and pommel.");
			return;
		}

		//Swordblank needs quenching and cannot be improved any more
		else if (quality == SWORDBLANK_QUENCH)
		{
			avatarSay("You have put as fine an edge on this blade as you can. It must now be quenched in water to harden the blade before the blank cools completely.");
			return;
		}

		//Swordblank can be worked, adjust its quality
		else
		{
			strength	= UI_get_npc_prop(AVATAR, STRENGTH);
			dexterity	= UI_get_npc_prop(AVATAR, DEXTERITY);

			fail_chance		= BASE_TEMPER_FAIL_CHANCE - (dexterity / 2);
			succeed_chance	= BASE_TEMPER_SUCCEED_CHANCE + (strength / 2);

			//Small chance (based on dexterity) that you fuck it up and mess up the blade
			if (quality > 0 && rand < fail_chance)
			{
				avatarBark(randomIndex(["\"Argh!\"", "\"Oops!\""]));

				//Give some party feedback
				if (UI_get_random(2) == 1) randomPartyBark(randomIndex(["\"Not so hard!\"", "\"Careful!\""]));

				//Lower the quality
				UI_set_item_quality(swordblank, quality - 1);
			}

			//otherwise, there's a bigger chance (based on strength) that you improve the quality of the blade
			else if (rand < succeed_chance)
			{
				//if this bumped the quality up to the next notch, notify the player they're making progress
				if (quality + 1 == SWORDBLANK_IMPROVED)
				{
					if (isNearby(MENION))
					{
						MENION.say("Menion comes over to check on your progress. \"Ah, thine blade is coming along nicely! A little more sweat and tears and thou shalt have a fine sword!\"");
						MENION.hide();
					}
					else avatarSay("After a short while you notice that the edge has definitely improved.");
				}

				//Otherwise, give some party feedback
				else if (UI_get_random(2) == 1) randomPartyBark(randomIndex(["\"Looks good!\"", "\"Keep it up!\"", "\"Put thine arm into it!\""]));

				UI_set_item_quality(swordblank, quality + 1);
			}
		}
	}
}